//--------------------------------------------------------------------------------------
// File: main.fx
//
// The effect file for the main sample.  
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
float4x4 g_mWorldViewProjection;	// World * View * Projection matrix

float    g_fTexSize;
float    g_fInvTexSize;

//-----------------------------------------------------------------------------
// Texture samplers
//-----------------------------------------------------------------------------
texture Texture;
sampler Sampler = 
sampler_state
{
    Texture = <Texture>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};
//-----------------------------------------------------------------------------
texture WangTexture;
sampler WangSampler = 
sampler_state
{
    Texture = <WangTexture>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
//-----------------------------------------------------------------------------
texture RandTexture;
sampler RandSampler = 
sampler_state
{
    Texture = <RandTexture>;
    MinFilter = POINT;
    MagFilter = POINT;
    MipFilter = NONE;

    AddressU = Wrap;
    AddressV = Wrap;
};


//-----------------------------------------------------------------------------
// Vertex shader output structure
//-----------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Position : POSITION;
    float2 TextureUV : TEXCOORD0;
};


//-----------------------------------------------------------------------------
// Name: WangVS
// Type: Vertex shader                                      
// Desc: standard transform and lighting.
//-----------------------------------------------------------------------------
VS_OUTPUT WangVS( float4 vPos : POSITION, 
                             float3 vNormal : NORMAL,
                             float2 vTexCoord0 : TEXCOORD0 )
{
    VS_OUTPUT Output;
  
    // tranform vertex position into screen space
    Output.Position = mul(vPos, g_mWorldViewProjection );
    
    // Just copy the texture coordinate through
    Output.TextureUV.xy = vTexCoord0;
  
    return Output;    
}


//-----------------------------------------------------------------------------
// Name: WangPS
// Type: Pixel shader
// Desc: This shader simply outputs the pixel's color 
//-----------------------------------------------------------------------------
float4 WangPS( VS_OUTPUT In ) : COLOR
{ 
	float2 TexCoord;
	float2 Tex;
	
	TexCoord.x = 0.25 * frac( g_fTexSize * In.TextureUV.x );
	TexCoord.y = 0.25 * frac( g_fTexSize * In.TextureUV.y );
	
	float2 e0 = tex2D( RandSampler, In.TextureUV ).xy;
	float ex = tex2D( RandSampler, In.TextureUV + float2( g_fInvTexSize, 0.0 ) ).x;
	float ey = tex2D( RandSampler, In.TextureUV + float2( 0.0, g_fInvTexSize ) ).y;
	
	if( e0.x < 0.5 )
	{
		if( ex < 0.5 )
		{
//			TexCoord.x += 0.0;
		}else{
			TexCoord.x += 0.25;
		}
	}else{
		if( ex < 0.5 )
		{
			TexCoord.x += 0.75;
		}else{
			TexCoord.x += 0.50;
		}
	}
	if( ey < 0.5 )
	{
		if( e0.y < 0.5 )
		{
			TexCoord.y += 0.75;
		}else{
			TexCoord.y += 0.50;
		}
	}else{
		if( e0.y < 0.5 )
		{
//			TexCoord.y += 0.0;
		}else{
			TexCoord.y += 0.25;
		}
	}

   return tex2D( WangSampler, TexCoord );
}


//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique WangTechnique
{
    pass P0
    {
        VertexShader = compile vs_1_1 WangVS();
        PixelShader  = compile ps_2_0 WangPS();
    }
}




//-----------------------------------------------------------------------------
// Name: WorldVertexShader
// Type: Vertex shader                                      
// Desc: standard transform and lighting.
//-----------------------------------------------------------------------------
VS_OUTPUT SimpleVS( float4 vPos : POSITION, 
                    float3 vNormal : NORMAL,
                    float2 vTexCoord0 : TEXCOORD0 )
{
    VS_OUTPUT Output;
  
    // tranform vertex position into screen space
    Output.Position = mul(vPos, g_mWorldViewProjection );
    
    // Just copy the texture coordinate through
    Output.TextureUV.xy = vTexCoord0;
    
    return Output;    
}


//-----------------------------------------------------------------------------
// Name: SimplePS
// Type: Pixel shader
// Desc: This shader simply outputs the pixel's color 
//-----------------------------------------------------------------------------
float4 SimplePS( VS_OUTPUT In ) : COLOR
{ 
    return tex2D( Sampler, In.TextureUV );
}


//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique SimpleTechnique
{
    pass P0
    {
        VertexShader = compile vs_1_1 SimpleVS();
        PixelShader  = compile ps_1_1 SimplePS();
    }
}




